@@ -4,7 +4,7 @@ use Moose;
 use IO::Handle;
 use Text::CSV;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 BEGIN {
   # One day, MooseX::Getopt will allow us to pass pass_through to Getopt::Long.
@@ -54,7 +54,7 @@ hasrw _init => (isa => 'Bool');
 # Normalized column indexes.
 hasrw columns => (isa => 'ArrayRef[Int]', cmd_aliases => 'c');
 
-# named fields
+# Named fields.
 hasrw fields => (isa => 'ArrayRef[Str]', cmd_aliases => 'f');
 
 # The input and output CSV processors.
@@ -136,9 +136,14 @@ sub _fields_to_columns {
   my @named_fields = grep { /\D/ } @all_fields;
 
   my @normalized_fields;
-  # if there is at least one named field, we need to read the header line from input and translate into column number
+  # If there is at least one named field, we need to read the header line from input and translate
+  # into column number.
   if (@named_fields) {
     my $header_map = $self->_get_header_map;
+    if (my @missing_fields = grep { not defined $header_map->{$_} } @named_fields) {
+        die "The following named fields aren't in the input header: ",
+            join(", ", @missing_fields), "\n";
+    }
     @normalized_fields = map { __normalize_column(/^\d+/ ? $_ : $header_map->{$_} ) } @all_fields;
   }
   else {
@@ -195,35 +200,32 @@ sub init {
   $self->_output_csv(Text::CSV->new({
       map { my $o = "output_$_"; $_ => $self->$o } keys %TextCSVOptions }));
 
-  # if columns aren't specified, look for --fields option this allows
+  # If columns aren't specified, look for the --fields option. It allows
   # use of named fields, list of comma-separated fields list, field
-  # ranges and so on
+  # ranges and so on.
 
   if (@columns) {
     warn "--fields (-f) option is ignored since columns are also specified." if $self->fields;
-  }
-  else {
+  } else {
     my @fields = $self->fields ? @{$self->fields} : ();
     if (@fields) {
       my $columns = $self->_fields_to_columns(\@fields);
       $self->columns($columns);
     }
-    # $self->columns($self->__fields_to_columns(\@fields)) if @fields;
   }
 }
 
 {
   my $line_read;
 
-  # read a line from input, but push it back for later reading
+  # Read a line from input, but push it back for later reading.
   sub _peek_line {
     my ($self) = @_;
 
     $line_read = $self->_input_csv->getline($self->_input_fh);
   }
 
-  # if there is a line already read, return it otherwise read from input
-
+  # If there is a line already read, return it; otherwise, read from input.
   sub _get_line {
     my ($self) = @_;
 
@@ -232,8 +234,7 @@ sub init {
       $line = $line_read;
       undef $line_read;
       return $line;
-    }
-    else {
+    } else {
       return $self->_input_csv->getline($self->_input_fh);
     }
   }
@@ -252,21 +253,38 @@ sub run {
       }
       
       if (!$self->_output_csv->print($self->_output_fh, $data)) {
-        warn $self->_output_csv->error_diag;
+        warn $self->format_error("Warning - Output error", diag => [$self->_input_csv->error_diag]), "\n";
         next INPUT;
       }
       $self->_output_fh->print("\n");
     }
 
-    # keeps us going on input errors.
+    # Keeps us going on input errors.
     # TODO: strict errors, according to command line, blah
     if (not defined $data) {
       last INPUT if $self->_input_csv->eof;
-      warn $self->_input_csv->error_diag;
+      warn $self->format_error("Warning - Input error", line => $., diag => [$self->_input_csv->error_diag]), "\n";
     }
   } }
 }
 
+sub format_error {
+    my $self = shift;
+    my $msg  = shift;
+    my %args = @_;
+
+    $msg .= ", line $args{line}"
+        if defined $args{line};
+
+    if ($args{diag}) {
+        my ($code, $err, $pos, $record) = @{ $args{diag} || [] };
+        $msg .= ": " . join " - ",
+            $code, $err, "position $pos",
+            (defined $record ? "record $record" : ());
+    }
+    return $msg;
+}
+
 1;
 
 __END__
@@ -278,4 +296,26 @@ App::CSV - process CSV files
 
 Please see L<csv>.
 
+=head1 COPYRIGHT (The "MIT" License)
+
+Copyright 2013 Gaal Yahas.
+
+Permission is hereby granted, free of charge, to any person obtaining a
+copy of this software and associated documentation files (the "Software"),
+to deal in the Software without restriction, including without limitation
+the rights to use, copy, modify, merge, publish, distribute, sublicense,
+and/or sell copies of the Software, and to permit persons to whom the
+Software is furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included
+in all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+OTHER DEALINGS IN THE SOFTWARE.
+
 =cut
